program icz19_TreeView;

{$R 'Resource.res' 'Resource.rc'}

uses
  Windows,
  Messages,
  CommCtrl;

const
  IDB_TREE = 4006;

const
  ClassName     = 'TreeViewWinClass';
  AppName       = 'Tree View Demo';
  TreeViewClass = 'SysTreeView32';
  Parent        = 'Parent Item';
  Child1        = 'child1';
  Child2        = 'child2';

var
  DragMode      : Boolean = False;
  hwndTreeView  : DWORD;
  hParent       : DWORD;
  hImageList    : DWORD;
  hDragImageList: DWORD;

function WndProc(hWnd: HWND; uMsg: UINT; wParam: WPARAM; lParam: LPARAM): LRESULT; stdcall;
var
  tvinsert : TTVInsertStruct;
  hBitmap  : DWORD;
  tvhit    : TTVHitTestInfo;
  HitHandle: THandle;
begin
  Result := 0;
  case uMsg of
    WM_CREATE:
      begin
        hwndTreeView := CreateWindowEx(0, TreeViewClass, nil, WS_CHILD or
          WS_VISIBLE or TVS_HASLINES or TVS_HASBUTTONS or TVS_LINESATROOT, 0, 0,
          200, 400, hWnd, 0, HInstance, nil);
        hImageList := ImageList_Create(16, 16, ILC_COLOR16, 2, 10);
        hBitmap := LoadBitmap(HInstance, PChar(IDB_TREE));
        ImageList_Add(hImageList, hBitmap, 0);
        DeleteObject(hBitmap);
        SendMessage(hwndTreeView, TVM_SETIMAGELIST, 0, hImageList);
        tvinsert.hParent      := nil;
        tvinsert.hInsertAfter := TVI_ROOT;
        tvinsert.item.mask    := TVIF_TEXT or TVIF_IMAGE or TVIF_SELECTEDIMAGE;
        tvinsert.item.pszText := Parent;
        tvinsert.item.iImage  := 0;
        tvinsert.item.iSelectedImage := 1;
        hParent := SendMessage(hwndTreeView, TVM_INSERTITEM, 0,
          Integer(@tvinsert));
        tvinsert.hParent      := Pointer(hParent);
        tvinsert.hInsertAfter := TVI_LAST;
        tvinsert.item.pszText := Child1;
        SendMessage(hwndTreeView, TVM_INSERTITEM, 0, Integer(@tvinsert));
        tvinsert.item.pszText := Child2;
        SendMessage(hwndTreeView, TVM_INSERTITEM, 0, Integer(@tvinsert));
      end;
    WM_MOUSEMOVE:
      if DragMode then
      begin
        tvhit.pt.x := LoWord(lParam); // horizontal position of the drag image
        tvhit.pt.y := HiWord(lParam); // vertical position
        ImageList_DragMove(LoWord(lParam), HiWord(lParam));
        ImageList_DragShowNolock(False);
        HitHandle := THandle(SendMessage(hwndTreeView, TVM_HITTEST, 0,
          Integer(@tvhit))); // check if an item is hit
        if HitHandle <> 0 then
          SendMessage(hwndTreeView, TVM_SELECTITEM, TVGN_DROPHILITE, HitHandle);
        ImageList_DragShowNolock(True);
      end;
    WM_LBUTTONUP:
      if DragMode then
      begin
        ImageList_DragLeave(hwndTreeView);
        ImageList_EndDrag;
        ImageList_Destroy(hDragImageList);
        SendMessage(hwndTreeView, TVM_SELECTITEM, TVGN_CARET,
          SendMessage(hwndTreeView, TVM_GETNEXTITEM, TVGN_DROPHILITE, 0)); // Get the currently hilited item
        SendMessage(hwndTreeView, TVM_SELECTITEM, TVGN_DROPHILITE, 0);
        ReleaseCapture;
        DragMode := False;
      end;
    WM_NOTIFY:
      with PNMTreeView(lParam)^ do
        if hdr.code = TVN_BEGINDRAG then
        begin
          hDragImageList := SendMessage(hwndTreeView, TVM_CREATEDRAGIMAGE, 0,
            Integer(itemNew.hItem));
          ImageList_BeginDrag(hDragImageList, 0, 0, 0);
          ImageList_DragEnter(hwndTreeView, ptDrag.x, ptDrag.y);
          SetCapture(hWnd);
          DragMode := True;
        end;
    WM_DESTROY:
      PostQuitMessage(0);
  else
    Result := DefWindowProc(hWnd, uMsg, wParam, lParam);
  end;
end;

var
  wc: TWndClassEx = (
    cbSize       : SizeOf(TWndClassEx);
    style        : CS_HREDRAW or CS_VREDRAW;
    lpfnWndProc  : @WndProc;
    cbClsExtra   : 0;
    cbWndExtra   : 0;
    hbrBackground: COLOR_APPWORKSPACE;
    lpszMenuName : nil;
    lpszClassName: ClassName;
    hIconSm      : 0;
  );
  msg: TMsg;

begin
  InitCommonControls;
  wc.hInstance := HInstance;
  wc.hIcon     := LoadIcon(0, IDI_APPLICATION);
  wc.hCursor   := LoadCursor(0, IDC_ARROW);
  RegisterClassEx(wc);
  CreateWindowEx(WS_EX_CLIENTEDGE, ClassName, AppName, WS_OVERLAPPED or
    WS_CAPTION or WS_SYSMENU or WS_MINIMIZEBOX or WS_MAXIMIZEBOX or WS_VISIBLE,
    Integer(CW_USEDEFAULT), Integer(CW_USEDEFAULT), 200, 400, 0, 0, HInstance,
    nil);
  while True do
  begin
    if not GetMessage(msg, 0, 0, 0) then
      Break;
    TranslateMessage(msg);
    DispatchMessage(msg);
  end;
  ExitCode := msg.wParam;
end.